@@ -62,6 +62,8 @@ def filters(dbname, exception_type, regex):
 @filters("mysql", sqla_exc.DatabaseError,
          r"^.*\b1205\b.*Lock wait timeout exceeded.*")
 @filters("mysql", sqla_exc.InternalError, r"^.*\b1213\b.*Deadlock found.*")
+@filters("mysql", sqla_exc.InternalError,
+         r"^.*\b1213\b.*detected deadlock/conflict.*")
 @filters("postgresql", sqla_exc.OperationalError, r"^.*deadlock detected.*")
 @filters("postgresql", sqla_exc.DBAPIError, r"^.*deadlock detected.*")
 @filters("ibm_db_sa", sqla_exc.DBAPIError, r"^.*SQL0911N.*")
@@ -385,6 +387,7 @@ def _raise_operational_errors_directly_filter(operational_error,
 
 @filters("mysql", sqla_exc.OperationalError, r".*\(.*(?:2002|2003|2006|2013|1047)")  # noqa
 @filters("mysql", sqla_exc.InternalError, r".*\(.*(?:1927)")  # noqa
+@filters("mysql", sqla_exc.InternalError, r".*Packet sequence number wrong")  # noqa
 @filters("postgresql", sqla_exc.OperationalError, r".*could not connect to server")  # noqa
 @filters("ibm_db_sa", sqla_exc.OperationalError, r".*(?:30081)")
 def _is_db_connection_error(operational_error, match, engine_name,
