@@ -32,6 +32,7 @@ import sqlalchemy.types as types
 
 from oslo_db._i18n import _LE
 from oslo_db import exception as exc
+from oslo_db.sqlalchemy import provision
 from oslo_db.sqlalchemy import utils
 
 LOG = logging.getLogger(__name__)
@@ -166,7 +167,7 @@ class WalkVersionsMixin(object):
 
             If we have ordered migrations 123abc, 456def, 789ghi and we run
             upgrading with the `snake_walk` argument set to `True`, the
-            migrations will be applied in the following order:
+            migrations will be applied in the following order::
 
                 `123abc => 456def => 123abc =>
                  456def => 789ghi => 456def => 789ghi`
@@ -185,7 +186,8 @@ class WalkVersionsMixin(object):
                                                        self.REPOSITORY))
 
         LOG.debug('latest version is %s', self.REPOSITORY.latest)
-        versions = range(self.INIT_VERSION + 1, self.REPOSITORY.latest + 1)
+        versions = range(int(self.INIT_VERSION) + 1,
+                         int(self.REPOSITORY.latest) + 1)
 
         for version in versions:
             # upgrade -> downgrade -> upgrade
@@ -267,7 +269,7 @@ class WalkVersionsMixin(object):
                 check = getattr(self, "_check_%03d" % version, None)
                 if check:
                     check(self.migrate_engine, data)
-        except exc.DbMigrationError:
+        except exc.DBMigrationError:
             msg = _LE("Failed to migrate to version %(ver)s on engine %(eng)s")
             LOG.error(msg, {"ver": version, "eng": self.migrate_engine})
             raise
@@ -434,7 +436,19 @@ class ModelsMigrationsSync(object):
         if issubclass(type(meta_type), BOOLEAN_METADATA):
             return not issubclass(type(insp_type), BOOLEAN_SQL)
 
-        return None  # tells alembic to use the default comparison method
+        # Alembic <=0.8.4 do not contain logic of comparing Variant type with
+        # others.
+        if isinstance(meta_type, types.Variant):
+            orig_type = meta_col.type
+            impl_type = meta_type.load_dialect_impl(ctxt.dialect)
+            meta_col.type = impl_type
+            try:
+                return self.compare_type(ctxt, insp_col, meta_col, insp_type,
+                                         impl_type)
+            finally:
+                meta_col.type = orig_type
+
+        return ctxt.impl.compare_type(insp_col, meta_col)
 
     def compare_server_default(self, ctxt, ins_col, meta_col,
                                insp_def, meta_def, rendered_meta_def):
@@ -468,7 +482,10 @@ class ModelsMigrationsSync(object):
                 isinstance(meta_def.arg, expr.False_) and insp_def == "'0'"
             )
 
-        if isinstance(meta_col.type, sqlalchemy.Integer):
+        impl_type = meta_col.type
+        if isinstance(impl_type, types.Variant):
+            impl_type = impl_type.load_dialect_impl(bind.dialect)
+        if isinstance(impl_type, (sqlalchemy.Integer, sqlalchemy.BigInteger)):
             if meta_def is None or insp_def is None:
                 return meta_def != insp_def
             return meta_def.arg != insp_def.split("'")[1]
@@ -555,6 +572,20 @@ class ModelsMigrationsSync(object):
         return self.FKInfo((fk.parent.name,), fk.column.table.name,
                            (fk.column.name,))
 
+    def filter_metadata_diff(self, diff):
+        """Filter changes before assert in test_models_sync().
+
+        Allow subclasses to whitelist/blacklist changes. By default, no
+        filtering is performed, changes are returned as is.
+
+        :param diff: a list of differences (see `compare_metadata()` docs for
+                     details on format)
+        :returns: a list of differences
+
+        """
+
+        return diff
+
     def test_models_sync(self):
         # recent versions of sqlalchemy and alembic are needed for running of
         # this test, but we already have them in requirements
@@ -564,9 +595,10 @@ class ModelsMigrationsSync(object):
             self.skipTest('sqlalchemy>=0.8.4 and alembic>=0.6.3 are required'
                           ' for running of this test: %s' % e)
 
-        # drop all tables after a test run
-        self.addCleanup(functools.partial(self.db.backend.drop_all_objects,
-                                          self.get_engine()))
+        # drop all objects after a test run
+        engine = self.get_engine()
+        backend = provision.Backend(engine.name, engine.url)
+        self.addCleanup(functools.partial(backend.drop_all_objects, engine))
 
         # run migration scripts
         self.db_sync(self.get_engine())
@@ -580,8 +612,8 @@ class ModelsMigrationsSync(object):
             mc = alembic.migration.MigrationContext.configure(conn, opts=opts)
 
             # compare schemas and fail with diff, if it's not empty
-            diff = alembic.autogenerate.compare_metadata(mc,
-                                                         self.get_metadata())
+            diff = self.filter_metadata_diff(
+                alembic.autogenerate.compare_metadata(mc, self.get_metadata()))
             if diff:
                 msg = pprint.pformat(diff, indent=2, width=20)
                 self.fail(
