@@ -938,6 +938,14 @@ class TestDeadlock(TestsExceptionFilter):
             "transaction')"
         )
 
+    def test_mysql_pymysql_wsrep_deadlock(self):
+        self._run_deadlock_detect_test(
+            "mysql",
+            "(1213, 'WSREP detected deadlock/conflict and aborted the "
+            "transaction. Try restarting the transaction')",
+            orig_exception_cls=self.InternalError
+        )
+
     def test_mysql_pymysql_galera_deadlock(self):
         self._run_deadlock_detect_test(
             "mysql",
@@ -1187,6 +1195,14 @@ class TestDBDisconnected(TestsExceptionFilter):
                 is_disconnect=False
             )
 
+    def test_packet_sequence_wrong_error(self):
+        self._test_ping_listener_disconnected(
+            "mysql",
+            self.InternalError(
+                'Packet sequence number wrong - got 35 expected 1'),
+            is_disconnect=False
+        )
+
     def test_mysql_ping_listener_disconnected(self):
         for code in [2006, 2013, 2014, 2045, 2055]:
             self._test_ping_listener_disconnected(
