@@ -18,11 +18,13 @@
 """Unit tests for SQLAlchemy specific code."""
 
 import logging
+import os
 
 import fixtures
 import mock
 from oslo_config import cfg
 from oslotest import base as oslo_test
+import six
 import sqlalchemy
 from sqlalchemy import Column, MetaData, Table
 from sqlalchemy.engine import url
@@ -31,9 +33,11 @@ from sqlalchemy.ext.declarative import declarative_base
 
 from oslo_db import exception
 from oslo_db import options as db_options
+from oslo_db.sqlalchemy import enginefacade
+from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session
-from oslo_db.sqlalchemy import test_base
+from oslo_db.tests.sqlalchemy import base as test_base
 
 
 BASE = declarative_base()
@@ -62,8 +66,8 @@ class RegexpFilterTestCase(test_base.DbTestCase):
         self.addCleanup(test_table.drop)
 
     def _test_regexp_filter(self, regexp, expected):
-        _session = self.sessionmaker()
-        with _session.begin():
+        with enginefacade.writer.using(test_base.context):
+            _session = test_base.context.session
             for i in ['10', '20', u'♥']:
                 tbl = RegexpTable()
                 tbl.update({'bar': i})
@@ -71,7 +75,7 @@ class RegexpFilterTestCase(test_base.DbTestCase):
 
         regexp_op = RegexpTable.bar.op('REGEXP')(regexp)
         result = _session.query(RegexpTable).filter(regexp_op).all()
-        self.assertEqual([r.bar for r in result], expected)
+        self.assertEqual(expected, [r.bar for r in result])
 
     def test_regexp_filter(self):
         self._test_regexp_filter('10', ['10'])
@@ -167,21 +171,21 @@ class SQLiteSavepointTest(test_base.DbTestCase):
         )
 
 
-class FakeDBAPIConnection():
+class FakeDBAPIConnection(object):
     def cursor(self):
         return FakeCursor()
 
 
-class FakeCursor():
+class FakeCursor(object):
     def execute(self, sql):
         pass
 
 
-class FakeConnectionProxy():
+class FakeConnectionProxy(object):
     pass
 
 
-class FakeConnectionRec():
+class FakeConnectionRec(object):
     pass
 
 
@@ -195,7 +199,7 @@ class ProgrammingError(Exception):
 
 class FakeDB2Engine(object):
 
-    class Dialect():
+    class Dialect(object):
 
         def is_disconnect(self, e, *args):
             expected_error = ('SQL30081N: DB2 Server connection is no longer '
@@ -216,7 +220,7 @@ class MySQLDefaultModeTestCase(test_base.MySQLOpportunisticTestCase):
                 "SHOW VARIABLES LIKE 'sql_mode'"
             ).first()[1]
 
-        self.assertTrue("TRADITIONAL" in sql_mode)
+        self.assertIn("TRADITIONAL", sql_mode)
 
 
 class MySQLModeTestCase(test_base.MySQLOpportunisticTestCase):
@@ -312,8 +316,8 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         self.assertFalse(ses.autocommit)
         self.assertTrue(ses.expire_on_commit)
 
-    @mock.patch('oslo_db.sqlalchemy.session.get_maker')
-    @mock.patch('oslo_db.sqlalchemy.session.create_engine')
+    @mock.patch('oslo_db.sqlalchemy.orm.get_maker')
+    @mock.patch('oslo_db.sqlalchemy.engines.create_engine')
     def test_creation_from_config(self, create_engine, get_maker):
         conf = cfg.ConfigOpts()
         conf.register_opts(db_options.database_opts, group='database')
@@ -346,6 +350,9 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
             sqlite_synchronous=mock.ANY,
             pool_timeout=mock.ANY,
             thread_checkin=mock.ANY,
+            json_serializer=None,
+            json_deserializer=None,
+            logging_name=mock.ANY,
         )
         get_maker.assert_called_once_with(engine=create_engine(),
                                           autocommit=False,
@@ -397,15 +404,15 @@ class SQLiteConnectTest(oslo_test.BaseTestCase):
     def test_sqlite_fk_listener(self):
         engine = self._fixture(sqlite_fk=True)
         self.assertEqual(
-            engine.scalar("pragma foreign_keys"),
-            1
+            1,
+            engine.scalar("pragma foreign_keys")
         )
 
         engine = self._fixture(sqlite_fk=False)
 
         self.assertEqual(
-            engine.scalar("pragma foreign_keys"),
-            0
+            0,
+            engine.scalar("pragma foreign_keys")
         )
 
     def test_sqlite_synchronous_listener(self):
@@ -414,15 +421,15 @@ class SQLiteConnectTest(oslo_test.BaseTestCase):
         # "The default setting is synchronous=FULL." (e.g. 2)
         # http://www.sqlite.org/pragma.html#pragma_synchronous
         self.assertEqual(
-            engine.scalar("pragma synchronous"),
-            2
+            2,
+            engine.scalar("pragma synchronous")
         )
 
         engine = self._fixture(sqlite_synchronous=False)
 
         self.assertEqual(
-            engine.scalar("pragma synchronous"),
-            0
+            0,
+            engine.scalar("pragma synchronous")
         )
 
 
@@ -433,12 +440,12 @@ class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
 
     def _assert_sql_mode(self, engine, sql_mode_present, sql_mode_non_present):
         mode = engine.execute("SHOW VARIABLES LIKE 'sql_mode'").fetchone()[1]
-        self.assertTrue(
-            sql_mode_present in mode
+        self.assertIn(
+            sql_mode_present, mode
         )
         if sql_mode_non_present:
-            self.assertTrue(
-                sql_mode_non_present not in mode
+            self.assertNotIn(
+                sql_mode_non_present, mode
             )
 
     def test_set_mode_traditional(self):
@@ -495,7 +502,7 @@ class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
                 )
             )
         ):
-            session._init_events.dispatch_on_drivername("mysql")(test_engine)
+            engines._init_events.dispatch_on_drivername("mysql")(test_engine)
 
             test_engine.raw_connection()
         self.assertIn('Unable to detect effective SQL mode',
@@ -556,95 +563,105 @@ class CreateEngineTest(oslo_test.BaseTestCase):
         self.args = {'connect_args': {}}
 
     def test_queuepool_args(self):
-        session._init_connection_args(
-            url.make_url("mysql://u:p@host/test"), self.args,
+        engines._init_connection_args(
+            url.make_url("mysql+pymysql://u:p@host/test"), self.args,
             max_pool_size=10, max_overflow=10)
-        self.assertEqual(self.args['pool_size'], 10)
-        self.assertEqual(self.args['max_overflow'], 10)
+        self.assertEqual(10, self.args['pool_size'])
+        self.assertEqual(10, self.args['max_overflow'])
 
     def test_sqlite_memory_pool_args(self):
         for _url in ("sqlite://", "sqlite:///:memory:"):
-            session._init_connection_args(
+            engines._init_connection_args(
                 url.make_url(_url), self.args,
                 max_pool_size=10, max_overflow=10)
 
             # queuepool arguments are not peresnet
-            self.assertTrue(
-                'pool_size' not in self.args)
-            self.assertTrue(
-                'max_overflow' not in self.args)
+            self.assertNotIn(
+                'pool_size', self.args)
+            self.assertNotIn(
+                'max_overflow', self.args)
 
-            self.assertEqual(self.args['connect_args']['check_same_thread'],
-                             False)
+            self.assertEqual(False,
+                             self.args['connect_args']['check_same_thread'])
 
             # due to memory connection
-            self.assertTrue('poolclass' in self.args)
+            self.assertIn('poolclass', self.args)
 
     def test_sqlite_file_pool_args(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("sqlite:///somefile.db"), self.args,
             max_pool_size=10, max_overflow=10)
 
         # queuepool arguments are not peresnet
-        self.assertTrue('pool_size' not in self.args)
-        self.assertTrue(
-            'max_overflow' not in self.args)
+        self.assertNotIn('pool_size', self.args)
+        self.assertNotIn(
+            'max_overflow', self.args)
 
         self.assertFalse(self.args['connect_args'])
 
         # NullPool is the default for file based connections,
         # no need to specify this
-        self.assertTrue('poolclass' not in self.args)
+        self.assertNotIn('poolclass', self.args)
+
+    def _test_mysql_connect_args_default(self, connect_args):
+        if six.PY3:
+            self.assertEqual({'charset': 'utf8', 'use_unicode': 1},
+                             connect_args)
+        else:
+            self.assertEqual({'charset': 'utf8', 'use_unicode': 0},
+                             connect_args)
 
     def test_mysql_connect_args_default(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("mysql://u:p@host/test"), self.args)
-        self.assertEqual(self.args['connect_args'],
-                         {'charset': 'utf8', 'use_unicode': 0})
+        self._test_mysql_connect_args_default(self.args['connect_args'])
 
     def test_mysql_oursql_connect_args_default(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("mysql+oursql://u:p@host/test"), self.args)
-        self.assertEqual(self.args['connect_args'],
-                         {'charset': 'utf8', 'use_unicode': 0})
+        self._test_mysql_connect_args_default(self.args['connect_args'])
+
+    def test_mysql_pymysql_connect_args_default(self):
+        engines._init_connection_args(
+            url.make_url("mysql+pymysql://u:p@host/test"), self.args)
+        self.assertEqual({'charset': 'utf8'}, self.args['connect_args'])
 
     def test_mysql_mysqldb_connect_args_default(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("mysql+mysqldb://u:p@host/test"), self.args)
-        self.assertEqual(self.args['connect_args'],
-                         {'charset': 'utf8', 'use_unicode': 0})
+        self._test_mysql_connect_args_default(self.args['connect_args'])
 
     def test_postgresql_connect_args_default(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("postgresql://u:p@host/test"), self.args)
-        self.assertEqual(self.args['client_encoding'], 'utf8')
+        self.assertEqual('utf8', self.args['client_encoding'])
         self.assertFalse(self.args['connect_args'])
 
     def test_mysqlconnector_raise_on_warnings_default(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url("mysql+mysqlconnector://u:p@host/test"),
             self.args)
-        self.assertEqual(self.args['connect_args']['raise_on_warnings'], False)
+        self.assertEqual(False, self.args['connect_args']['raise_on_warnings'])
 
     def test_mysqlconnector_raise_on_warnings_override(self):
-        session._init_connection_args(
+        engines._init_connection_args(
             url.make_url(
                 "mysql+mysqlconnector://u:p@host/test"
                 "?raise_on_warnings=true"),
             self.args
         )
 
-        self.assertFalse('raise_on_warnings' in self.args['connect_args'])
+        self.assertNotIn('raise_on_warnings', self.args['connect_args'])
 
     def test_thread_checkin(self):
         with mock.patch("sqlalchemy.event.listens_for"):
             with mock.patch("sqlalchemy.event.listen") as listen_evt:
-                session._init_events.dispatch_on_drivername(
+                engines._init_events.dispatch_on_drivername(
                     "sqlite")(mock.Mock())
 
         self.assertEqual(
             listen_evt.mock_calls[0][1][-1],
-            session._thread_yield
+            engines._thread_yield
         )
 
 
@@ -693,7 +710,7 @@ class PatchStacktraceTest(test_base.DbTestCase):
 
         with mock.patch("traceback.extract_stack", side_effect=extract_stack):
 
-            session._add_trace_comments(engine)
+            engines._add_trace_comments(engine)
             conn = engine.connect()
             orig_do_exec = engine.dialect.do_execute
             with mock.patch.object(engine.dialect, "do_execute") as mock_exec:
@@ -704,4 +721,5 @@ class PatchStacktraceTest(test_base.DbTestCase):
             call = mock_exec.mock_calls[0]
 
             # we're the caller, see that we're in there
-            self.assertIn("tests/sqlalchemy/test_sqlalchemy.py", call[1][1])
+            caller = os.path.join("tests", "sqlalchemy", "test_sqlalchemy.py")
+            self.assertIn(caller, call[1][1])
